

#include <vector>
#include <iostream>

#include "protocol.h"

typedef struct vertex3_ {
	float v[3];
} vertex3;


class Matrix;
class ColumnVector;

using namespace std;

class SkeletonCalibration
{
public:
	/// constructs an empty skeleton
	/// @param n number of sensors
	/// @param bufferSize size of the buffer of motion data
	SkeletonCalibration(int n, int initialBufferSize=10000);
	~SkeletonCalibration();

	// sets the root of the skeleton tree
	/// @param r index of the root node (default: 0)	
	void setRoot(int r);
	int guessRoot();

	/// add new data the the buffer
	/// ISSUE: As a queue has no iterators, we must do double copying!
	void addData(vector<vector<station_t> > *data);

	// performs the calbration
	void calibrateSkeleton();

	// get the results
	int* getParenthood();
	int getRoot();
	float* getLinkageGraph();

	//TODO: these methods should recalculate the joint positions only when necessary
	vector<vertex3> getJoints(vector<station_t> &frame);
	vector<vertex3> getVertices(vector<station_t> &frame);
	vector<float>   getRelativeErrors(vector<station_t> &frame);
	
	vector<int> getEdges();

	void writeCalibrationResult(ostream &stream);
	void readCalibrationResult(istream &stream);

//	Matrix quat2rot(float *q);                   // moved to separate file
//	float* rot2quat(const Matrix a, float *q);

private:
	int nSensors;  ///< number of sensors
	int root;      ///< root of the skeleton tree

	// utility functions
	void prim(float *G, int *p, int n, int root);
	void degree(int *tree, int *deg, int n);
//	inline Matrix quat2rot(float *q);

	//core functions
	
	/// constructs the linear equation system out of the data	
	void makeCalibrationMatrix(int n, Matrix **M, ColumnVector **d, int strive=1);
	/// solves the linear equation system for two joints
	ColumnVector solveCalibration(Matrix &Q1, Matrix &Q2, ColumnVector &d1, ColumnVector &d2);

	vector<vector<station_t> > *stationBuffer;  ///< buffer for sensor (station) data

	// data structures used DURING calibration
	Matrix **MM;        ///< array of n matrices representing the rotations of the sensors
	ColumnVector **dd;  ///< array of n column vectors representing the translation of the sensors
	float *graph;       ///< residual errors for joint hypotheses

	// data structures which are RESULT of the calibration
	int *p;
	float *residualRMS;
	float *residualMax;
	float *residualAbs;

	ColumnVector *L;
	ColumnVector *C;

	// animation part

	// utility functions for animation
	void calculateJoints(vector<station_t> &frame);

	// temporary variables for animation
	ColumnVector *joint;
	ColumnVector *joint1;
	ColumnVector *joint2;
	ColumnVector *residual;
};
