#include "SkeletonCalibration.hh"
#include "SkeletonEnv.hh"

#include <iostream>
#include <fstream>


int main(int argc, char *argv[])
{
	if(argc < 2) {
		cerr << "syntax: calc-joints logfile [calfile]" << endl;
		cerr << "calculate the joint position at each point in time and write them to stdout" << endl;
		cerr << "if no calfile is specified, the calibration is done from the logfile" << endl;

		return 0;
	}

	vector<vector<station_t> > *data = readFromFile(argv[1]);

	int n = (*data)[data->size() / 2].size(); // inspect a frame in the middle
	cerr << "found " << data->size() << " frames with " << n << " sensors" << endl;

	SkeletonCalibration *skel = new SkeletonCalibration(n,data->size());

	if(argc >= 3) {
		ifstream file(argv[2]);
		skel->readCalibrationResult(file);
		file.close();
	} else {
		skel->addData(data);
		skel->guessRoot();
		skel->calibrateSkeleton();
	}

	for(unsigned int i=0; i < data->size(); i++) {
		vector<station_t> frame = data->at(i);
		vector<vertex3> joint = skel->getVertices(frame);

		// joint[0] .. joint[n-1] are the sensor positions
		// joint[n] .. joint[2n-2] are the joint positions

		for(int j=n; j < joint.size(); j++) {
			station_t s = frame[j-n];
			cout << s.timestamp << " " << s.framecount << " " << (int) s.head.station << " 0 "
			     << joint[j].v[0] << " " << joint[j].v[1] << " " << joint[j].v[2] << " 1 0 0 0" << endl;
		}
	}
	
	delete skel;
	delete data;
}
