inline Matrix quat2rot(float *q)
{
	Matrix rot(3,3);

	rot(1,1) = q[0]*q[0] + q[1]*q[1] - q[2]*q[2] - q[3]*q[3];
	rot(1,2) = 2*(q[1]*q[2] - q[0]*q[3]);
	rot(1,3) = 2*(q[1]*q[3] + q[0]*q[2]);

	rot(2,1) = 2*(q[1]*q[2] + q[0]*q[3]);
	rot(2,2) = q[0]*q[0] - q[1]*q[1] + q[2]*q[2] - q[3]*q[3];
	rot(2,3) = 2*(q[2]*q[3] - q[0]*q[1]);

	rot(3,1) = 2*(q[1]*q[3] - q[0]*q[2]);
	rot(3,2) = 2*(q[2]*q[3] + q[0]*q[1]);
	rot(3,3) = q[0]*q[0] - q[1]*q[1] - q[2]*q[2] + q[3]*q[3];

	return rot;
}


inline float* rot2quat(const Matrix a,float *q)
{
	const float fEps = std::numeric_limits<float>::epsilon();
	
	double trace = a(1,1) + a(2,2) + a(3,3);
	if(trace > fEps) {
	//if(true) {
		double s = 0.5 / sqrt(trace+1.0);
		q[0] = 0.25 / s;
		q[1] = ( a(3,2) - a(2,3) ) * s;
		q[2] = ( a(1,3) - a(3,1) ) * s;
		q[3] = ( a(2,1) - a(1,2) ) * s;
	} else {
		if ( a(1,1) > a(2,2) && a(1,1) > a(3,3) ) {
			double s = 0.5 / sqrt( 1.0 + a(1,1) - a(2,2) - a(3,3));
			q[0] = (a(3,2) - a(2,3) ) * s;
			q[1] = 0.25 / s;
			q[2] = (a(1,2) + a(2,1) ) * s;
			q[3] = (a(1,3) + a(3,1) ) * s;
		} else if (a(2,2) > a(3,3)) {
			double s = 0.5 / sqrt( 1.0 + a(2,2) - a(1,1) - a(3,3));
			q[0] = (a(1,3) - a(3,1) ) * s;
			q[1] = (a(1,2) + a(2,1) ) * s;
			q[2] = 0.25 / s;
			q[3] = (a(2,3) + a(3,2) ) * s;
		} else {
			double s = 0.5 / sqrt( 1.0 + a(3,3) - a(1,1) - a(2,2) );
			q[0] = (a(2,1) - a(1,2) ) * s;
			q[1] = (a(1,3) + a(3,1) ) * s;
			q[2] = (a(2,3) + a(3,2) ) * s;
			q[3] = 0.25 / s;
		}
	}
	return q;
}

/* old version (buggy?!)
inline float* rot2quat(const Matrix a,float *q)
{
	const float fEps = std::numeric_limits<float>::epsilon();
	
	double trace = a(1,1) + a(2,2) + a(3,3) + 1.0;
	if( trace > fEps *100) {
	//if(true) {
		double s = 0.5/ sqrt(trace);
		q[0] = 0.25 / s;
		q[1] = ( a(3,2) - a(2,3) ) * s;
		q[2] = ( a(1,3) - a(3,1) ) * s;
		q[3] = ( a(2,1) - a(1,2) ) * s;
	} else {
		if ( a(1,1) > a(2,2) && a(1,1) > a(3,3) ) {
			double s = 2.0 * sqrt( 1.0 + a(1,1) - a(2,2) - a(3,3));
			q[0] = (a(2,3) - a(3,2) ) / s;
			q[1] = 0.25 * s;
			q[2] = (a(1,2) + a(2,1) ) / s;
			q[3] = (a(1,3) + a(3,1) ) / s;
		} else if (a(2,2) > a(3,3)) {
			double s = 2.0 * sqrt( 1.0 + a(2,2) - a(1,1) - a(3,3));
			q[0] = (a(1,3) - a(3,1) ) / s;
			q[1] = (a(1,2) + a(2,1) ) / s;
			q[2] = 0.25 * s;
			q[3] = (a(2,3) + a(3,2) ) / s;
		} else {
			double s = 2.0 * sqrt( 1.0 + a(3,3) - a(1,1) - a(2,2) );
			q[0] = (a(1,2) - a(2,1) ) / s;
			q[1] = (a(1,3) + a(3,1) ) / s;
			q[2] = (a(2,3) + a(3,2) ) / s;
			q[3] = 0.25 * s;
		}
	}
	return q;
}
*/
