#include "SkeletonCalibration.hh"
#include "SkeletonEnv.hh"

#include <iostream>
#include <fstream>

#include <newmatap.h>
#include <math.h>
#include "quaternion.h"

int main(int argc, char *argv[])
{
	if(argc < 2) {
		cerr << "syntax: transform-data logfile [root node]" << endl;
		cerr << "transform sensor data into the coordinate frame of the root node" << endl;
		cerr << " if no root is supplied, the least moving sensor is used" << endl;

		return 0;
	}

	vector<vector<station_t> > *data = readFromFile(argv[1]);
	vector<vector<station_t> > data_cropped;

	int n = (*data)[data->size() / 2].size(); // inspect a frame in the middle

	cerr << "found " << data->size() << " frames with " << n << " sensors" << endl;

	SkeletonCalibration skel(n,25000);

	int root=0;

	if(argc >= 3) {
		root = atol(argv[3]);
	} else {
		skel.addData(data);
		skel.guessRoot();
		root = skel.getRoot();
	}
	
	cerr << "root is " << root << endl;

	for(unsigned int i=0; i < data->size(); i++) {
		station_t r = data->at(i).at(root);
		
		Matrix R = quat2rot(r.quaternion);
		ColumnVector t(3);
		t << r.x << r.y << r.z;
		//cerr << "x" << endl;
		for(int j=0; j < n; j++) {
			station_t point = data->at(i).at(j);
			ColumnVector p(3);
			p << point.x << point.y << point.z;

			ColumnVector p_t = R.t()*(p-t);
			
			Matrix R_point = quat2rot(point.quaternion);
			
			rot2quat(R.t()*R_point, point.quaternion);
			
			point.x = p_t(1);
			point.y = p_t(2);
			point.z = p_t(3);
			
			cout << point;
		}
	}

	delete data;
}
