// Shows calibrated motion data in two views

#include "SkeletonCalibration.hh"
#include "SkeletonViewer.hh"
#include "SkeletonEnv.hh"

#include <iostream>
#include <iomanip>
#include <fstream>

#include <newmatap.h>
#include <newmatio.h>

#include <math.h>
#include <time.h>

#include <GL/glu.h>

#include <stdio.h>

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Value_Output.H>
#include <FL/Fl_Slider.H>

#include <FL/gl.h>
#include <FL/Fl_Gl_Window.H>

/**
command line: fltk-skeleton logfile [calfile]
 if no calfile is specified, the calibration is done from the logfile

keys:
 c - print coordinates of all sensors to cout

*/


class SkeletonWidget : public Fl_Gl_Window {
	void draw();
//	int handle(int);
public:
	SkeletonWidget(int X, int Y, int W, int H, const char *L)
	: Fl_Gl_Window(X, Y, W, H, L) {}

	void setView(float eyeX,    float eyeY,    float eyeZ,
	             float centerX, float centerY, float centerZ,
	             float upX,     float upY,     float upZ);
	void setViewer(SkeletonViewer *viewer);
private:
	float ex,ey,ez,cx,cy,cz,ux,uy,uz;
	SkeletonViewer *viewer;
};


class SkeletonUI {
public:
  SkeletonUI();
  ~SkeletonUI();
  Fl_Double_Window* create(const char *caption);
  void init(const char *caption);
  void main();
private:
  void cb_Play(Fl_Button*, void*);
    static void cb_Play_s(Fl_Button* o, void* v) {((SkeletonUI*)(o->parent()->user_data()))->cb_Play(o,v);}
  void cb_Slider(Fl_Slider*, void* v);
    static void cb_Slider_s(Fl_Slider* o, void* v) {((SkeletonUI*)(o->parent()->user_data()))->cb_Slider(o,v);}
  void cb_Window(Fl_Window* o, void* v);
    static void cb_Window_s(Fl_Window* o, void* v) {instance->cb_Window(o,v);} // window was already destroyed!
  int handle(int event);
    static int handle_s(int event) { instance->handle(event); }

  void showFrame();
  static SkeletonUI *instance;

  SkeletonWidget *canvas;
  SkeletonWidget *canvas2;
  Fl_Value_Output *framecount;
  Fl_Slider* frameslider;
  Fl_Button* play;
  Fl_Window* wnd;

  bool playing;
  bool exiting;

  unsigned int currentFrameNum;
public:
  SkeletonCalibration *skeleton;
  SkeletonViewer *viewer;
  vector<vector<station_t> > *data;
  
};

SkeletonUI *SkeletonUI::instance=0;

void SkeletonWidget::setView(float eyeX,    float eyeY,    float eyeZ,
                             float centerX, float centerY, float centerZ,
                             float upX,     float upY,     float upZ)
{
    ex = eyeX;    ey = eyeY;    ez = eyeZ;
    cx = centerX; cy = centerY; cz = centerZ;
    ux = upX;     uy = upY;     uz = upZ;
}


void SkeletonWidget::setViewer(SkeletonViewer *viewer)
{
	this->viewer = viewer;
}


void SkeletonWidget::draw() {
    if (!valid()) {
		//... set up projection, viewport, etc ...
		//... window size is in w() and h().
		//... valid() is turned on by FLTK after draw() returns
		
		viewer->glInit();
	
		glClearColor(0.4, 0.4, 0.4, 0.0);
		
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(85.654, w()/h(), 1.0, 200.0);
		gluLookAt(ex, ey, ez,
				cx, cy, cz,
				ux, uy, uz);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
    }
    // ... draw ...
//    glDrawBuffer(GL_FRONT_AND_BACK);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    viewer->glDraw();
//    glDrawBuffer(GL_BACK);
}
/*
int SkeletonWidget::handle(int event) {
  switch(event) {
  case FL_PUSH:
    //... mouse down event ...
    //... position in Fl::event_x() and Fl::event_y()
    return 1;
  case FL_DRAG:
    //... mouse moved while down event ...
    return 1;
  case FL_RELEASE:   
    //... mouse up event ...
    return 1;
  case FL_FOCUS :
  case FL_UNFOCUS :
    //... Return 1 if you want keyboard events, 0 otherwise
    return 1;
  case FL_KEYBOARD:
	  cout << "key: " << Fl::event_key() << endl;
    //... keypress, key is in Fl::event_key(), ascii in Fl::event_text()
    //... Return 1 if you understand/use the keyboard event, 0 otherwise...
    return 1;
  case FL_SHORTCUT:
    //... shortcut, key is in Fl::event_key(), ascii in Fl::event_text()
    //... Return 1 if you understand/use the shortcut event, 0 otherwise...
    return 1;
  default:
    // pass other events to the base class...
    return Fl_Gl_Window::handle(event);
  }
}*/

SkeletonUI::SkeletonUI() {
	currentFrameNum = 0;
	wnd=0;
	instance=this;
	playing=true;
	exiting=false;
}

SkeletonUI::~SkeletonUI() {
	if(wnd) delete wnd;
}

void SkeletonUI::cb_Slider(Fl_Slider* o, void* v) {
	currentFrameNum = (unsigned int) o->value();
	showFrame();
}


void SkeletonUI::cb_Window(Fl_Window* o, void* v)
{
	exiting=true;
}


void SkeletonUI::cb_Play(Fl_Button* o, void* v)
{
	if(playing) {
		o->label("@||");
		playing=false;
	} else {
		o->label("@>");
		playing=true;
	}
}


int SkeletonUI::handle(int event)
{
	switch(event) {
		case FL_SHORTCUT:
			//cout << "key: " << (char) Fl::event_key() << endl;
			switch(Fl::event_key()) {
				case 'c':
					vector<station_t> frame = (*data)[currentFrameNum];
					cout << "frame " << currentFrameNum << endl;
					for(int i=0; i < frame.size(); i++) {
						cout << i << "   " << frame[i].x << " " << frame[i].y << " " << frame[i].z << endl;
					} cout << endl;
					return 1;
			}
			return 0;
		default:
			return 0;
	}
}

Fl_Double_Window* SkeletonUI::create(const char *caption) {
  Fl_Double_Window* w;
  { Fl_Double_Window* o = new Fl_Double_Window(830, 455, caption);
    w = o;
    o->user_data((void*)(this));
    { canvas = new SkeletonWidget(10, 10, 400, 400, "View Side");
    } // Fl_Gl_Window* canvas
	{ canvas2 = new SkeletonWidget(420, 10, 400, 400, "View Top");
    } // Fl_Gl_Window* canvas
    { framecount = new Fl_Value_Output(730, 420, 100, 25);
      framecount->box(FL_FLAT_BOX);
      framecount->textsize(24);
    } // Fl_Value_Output* framecount
    { frameslider = new Fl_Slider(45, 420, 660, 25);
      frameslider->box(FL_FLAT_BOX);
      frameslider->type(FL_HOR_NICE_SLIDER);
	  frameslider->step(1);
      frameslider->callback((Fl_Callback*)cb_Slider_s);
    } // Fl_Slider* frameslider
    { play = new Fl_Button(10, 420, 25, 25, "@>");
      play->callback((Fl_Callback*)cb_Play_s);
    } // Fl_Button* o
    o->end();
	o->callback((Fl_Callback*)cb_Window_s);
  } // Fl_Double_Window* o
  return w;
}

void SkeletonUI::init(const char *caption)
{
	wnd = create(caption);

	canvas->setViewer(viewer);
	canvas->setView(45.0, -100.0,    -20.0,
	                45.0,   0.0,    -20.0,
	                 0.0,   0.0,   -1.0);

	canvas2->setViewer(viewer);
	canvas2->setView(45.0,  0.0, -130.0,
	                 45.0,  0.0,    0.0,
	                 -1.0,  0.0,    0.0);

	frameslider->minimum(0);
	frameslider->maximum(data->size()-1);

	Fl::add_handler(handle_s);

	wnd->show();
}

void SkeletonUI::showFrame()
{
	viewer->setFrame( (*data)[currentFrameNum] );
	canvas->redraw();
	canvas2->redraw();
	framecount->value(currentFrameNum);
	frameslider->value(currentFrameNum);
}

void SkeletonUI::main()
{
	while(!exiting) {
		usleep(1000);
		if(playing && currentFrameNum < data->size()-1) {
			currentFrameNum++;
			showFrame();
		}
		Fl::check();
	}
}


int main(int argc, char *argv[])
{
	if(argc < 2) {
		cerr << "syntax: fltk-skeleton logfile [calfile]" << endl;
		cerr << endl << "if no calfile is specified, the calibration is done from the logfile" << endl;
		return 0;
	}

	vector<vector<station_t> > *data = readFromFile(argv[1]);
	
	int n = (*data)[data->size() / 2].size(); // inspect a frame in the middle
	cerr << "found " << data->size() << " frames with " << n << " sensors" << endl;

	SkeletonCalibration *skel = new SkeletonCalibration(n,data->size());
	SkeletonViewer *viewer = new SkeletonViewer(skel);

	char buffer[256], *caption;
	if(argc >= 3) {
		ifstream file(argv[2]);
		skel->readCalibrationResult(file);
		file.close();

		sprintf(buffer,"%s  -  (%s)",argv[1], argv[2]);
		caption=buffer;
	} else {
		skel->addData(data);
		skel->guessRoot();
		skel->calibrateSkeleton();

		caption = argv[1];
	}

	SkeletonUI *ui = new SkeletonUI();

	ui->skeleton = skel;
	ui->viewer = viewer;
	ui->data = data;

	ui->init(caption);

	ui->main();

	delete ui;
	delete viewer;
	delete skel;
	delete data;

//    return Fl::run();
}
